from typing import List, Optional, Union
from pydantic import BaseModel, Field, ValidationError
from dataclasses import dataclass
from producteca.abstract.abstract_dataclass import BaseService
from producteca.products.search_products import SearchProduct, SearchProductParams
from producteca.utils import clean_model_dump
import logging
import requests

_logger = logging.getLogger(__name__)


class Attribute(BaseModel):
    key: str
    value: str


class Tag(BaseModel):
    tag: str


class Dimensions(BaseModel):
    weight: Optional[float] = None
    width: Optional[float] = None
    height: Optional[float] = None
    length: Optional[float] = None
    pieces: Optional[int] = None


class Deal(BaseModel):
    campaign: str
    regular_price: Optional[float] = Field(default=None, alias='regularPrice')
    deal_price: Optional[float] = Field(default=None, alias='dealPrice')


class Stock(BaseModel):
    quantity: Optional[int] = None
    available_quantity: Optional[int] = Field(default=None, alias='availableQuantity')
    warehouse: Optional[str] = None
    warehouse_id: Optional[int] = Field(default=None, alias='warehouseId')
    reserved: Optional[int] = None
    available: Optional[int] = None


class Price(BaseModel):
    amount: Optional[float] = None
    currency: str
    price_list: str = Field(alias='priceList')
    price_list_id: Optional[int] = Field(default=None, alias='priceListId')


class Picture(BaseModel):
    url: str


class Integration(BaseModel):
    app: Optional[int] = None
    integration_id: Optional[str] = Field(default=None, alias='integrationId')
    permalink: Optional[str] = None
    status: Optional[str] = None
    listing_type: Optional[str] = Field(default=None, alias='listingType')
    safety_stock: Optional[int] = Field(default=None, alias='safetyStock')
    synchronize_stock: Optional[bool] = Field(default=None, alias='synchronizeStock')
    is_active: Optional[bool] = Field(default=None, alias='isActive')
    is_active_or_paused: Optional[bool] = Field(default=None, alias='isActiveOrPaused')
    id: Optional[int] = None
    parent_integration: Optional[str] = Field(default=None, alias='parentIntegration')


class Variation(BaseModel):
    variation_id: Optional[int] = Field(default=None, alias='variationId')
    components: Optional[List] = None
    pictures: Optional[Union[List[Picture], List]] = None
    stocks: Optional[Union[List[Stock], List]] = None
    attributes_hash: Optional[str] = Field(default=None, alias='attributesHash')
    primary_color: Optional[str] = Field(default=None, alias='primaryColor')
    thumbnail: Optional[str] = None
    attributes: Optional[Union[List[Attribute], List]] = None
    integrations: Optional[Union[List[Integration], List]] = None
    id: Optional[int] = None
    sku: Optional[str] = None
    barcode: Optional[str] = None


class MeliCategory(BaseModel):
    meli_id: Optional[str] = Field(default=None, alias='meliId')
    accepts_mercadoenvios: Optional[bool] = Field(default=None, alias='acceptsMercadoenvios')
    suggest: Optional[bool] = None
    fixed: Optional[bool] = None


class BundleComponent(BaseModel):
    quantity: int
    variation_id: int = Field(alias='variationId')
    product_id: int = Field(alias='productId')


class BundleVariation(BaseModel):
    variation_id: int = Field(alias='variationId')
    components: Union[List[BundleComponent], List]


class BundleResult(BaseModel):
    company_id: int = Field(alias='companyId')
    product_id: int = Field(alias='productId')
    variations: Union[List[BundleVariation], List]
    id: str


class BundleResponse(BaseModel):
    results: Union[List[BundleResult], List]
    count: int


class Product(BaseModel):
    integrations: Optional[Union[List[Integration], List]] = None
    variations: Optional[Union[List[Variation], List]] = None
    is_simple: Optional[bool] = Field(default=None, alias='isSimple')
    has_variations: Optional[bool] = Field(default=None, alias='hasVariations') 
    thumbnail: Optional[str] = None
    category: Optional[str] = None
    notes: Optional[str] = None
    prices: Optional[Union[List[Price], List]] = None
    buying_price: Optional[float] = Field(default=None, alias='buyingPrice')
    is_archived: Optional[bool] = Field(default=None, alias='isArchived')
    dimensions: Optional[Union[Dimensions, dict]] = None
    attributes: Optional[Union[List[Attribute], List]] = None
    metadata: Optional[List[str]] = None
    is_original: Optional[bool] = Field(default=None, alias='isOriginal')
    name: str
    code: Optional[str] = None
    sku: Optional[str] = None
    brand: Optional[str] = None
    id: Optional[int] = None


class ProductVariationBase(BaseModel):
    sku: str
    variation_id: Optional[int] = Field(default=None, alias='variationId')
    code: Optional[str] = None
    barcode: Optional[str] = None
    attributes: Union[List[Attribute], List] = []
    tags: Optional[List[str]] = []
    buying_price: Optional[float] = Field(default=None, alias='buyingPrice')
    dimensions: Optional[Union[Dimensions, dict]] = Field(default_factory=Dimensions)
    brand: Optional[str] = ''
    notes: Optional[str] = ''
    deals: Optional[Union[List[Deal], List]] = []
    stocks: Optional[Union[List[Stock], List]] = []
    prices: Optional[Union[List[Price], List]] = []
    pictures: Optional[Union[List[Picture], List]] = []


class ProductVariation(ProductVariationBase):
    category: Optional[str] = Field(default=None)
    name: Optional[str] = None


class Shipping(BaseModel):
    local_pickup: Optional[bool] = Field(default=None, alias='localPickup')
    mode: Optional[str] = None
    free_shipping: Optional[bool] = Field(default=None, alias='freeShipping')
    free_shipping_cost: Optional[float] = Field(default=None, alias='freeShippingCost')
    mandatory_free_shipping: Optional[bool] = Field(default=None, alias='mandatoryFreeShipping')
    free_shipping_method: Optional[str] = Field(default=None, alias='freeShippingMethod')


class MShopsShipping(BaseModel):
    enabled: Optional[bool] = None


class AttributeCompletion(BaseModel):
    product_identifier_status: Optional[str] = Field(default=None, alias='productIdentifierStatus')
    data_sheet_status: Optional[str] = Field(default=None, alias='dataSheetStatus')
    status: Optional[str] = None
    count: Optional[int] = None
    total: Optional[int] = None


class MeliProduct(BaseModel):
    product_id: Optional[int] = Field(default=None, alias='productId')
    tags: Optional[List[str]] = Field(default=None)
    has_custom_shipping_costs: Optional[bool] = Field(default=None, alias='hasCustomShippingCosts')
    shipping: Optional[Union[Shipping, dict]] = None
    mshops_shipping: Optional[Union[MShopsShipping, dict]] = Field(default=None, alias='mShopsShipping')
    add_free_shipping_cost_to_price: Optional[bool] = Field(default=None, alias='addFreeShippingCostToPrice')
    category: Union[MeliCategory, dict]
    attribute_completion: Optional[Union[AttributeCompletion, dict]] = Field(default=None, alias='attributeCompletion')
    catalog_products: Optional[List[str]] = Field(default=None, alias='catalogProducts')
    warranty: Optional[str] = None
    domain: Optional[str] = None
    listing_type_id: Optional[str] = Field(default=None, alias='listingTypeId')
    catalog_products_status: Optional[str] = Field(default=None, alias='catalogProductsStatus')


class ErrorMessage(BaseModel):
    en: str
    es: str
    pt: str


class ErrorReason(BaseModel):
    code: str
    error: str
    message: ErrorMessage
    data: Optional[dict] = None


class ResolvedValue(BaseModel):
    updated: bool


class ResolvedError(BaseModel):
    resolved: Optional[bool] = None
    reason: Optional[Union[ErrorReason, dict]] = None
    value: Optional[Union[ResolvedValue, dict]] = None
    statusCode: Optional[int] = None


class ErrorContext(BaseModel):
    _ns_name: str
    id: int
    requestId: str
    tokenAppId: str
    appId: str
    bearer: str
    eventId: str


class SynchronizeResponse(BaseModel):
    product: Optional[Union[ResolvedError, dict]] = None
    variation: Optional[Union[ResolvedError, dict]] = None
    deals: Optional[Union[ResolvedError, dict]] = None
    bundles: Optional[Union[ResolvedError, dict]] = None
    taxes: Optional[Union[ResolvedError, dict]] = None
    meliProductListingIntegrations: Optional[Union[ResolvedError, dict]] = None
    tags: Optional[Union[ResolvedError, dict]] = None
    productIntegrations: Optional[Union[ResolvedError, dict]] = None
    statusCode: Optional[int] = None
    error_context: Optional[Union[ErrorContext, dict]] = Field(None, alias='error@context')


class ListedSynchronizeResponse(BaseModel):
    results: Union[List[SynchronizeResponse], List]


@dataclass
class ProductService(BaseService):
    endpoint: str = 'products'
    create_if_it_doesnt_exist: bool = Field(default=False, exclude=True)

    def __call__(self, **payload):
        self._record = Product(**payload)
        return self

    def synchronize(self, payload) -> Union[Product, SynchronizeResponse]:
        endpoint_url = self.config.get_endpoint(f'{self.endpoint}/synchronize')
        headers = self.config.headers.copy()
        headers.update({"createifitdoesntexist": str(self.create_if_it_doesnt_exist).lower()})
        product_variation = ProductVariation(**payload)
        if not product_variation.code and not product_variation.sku:
            raise Exception("Sku or code should be provided to update the product")
        # Hacer model_dump con limpieza automática de valores vacíos
        data = clean_model_dump(product_variation)
        _logger.info(f"Synchronizing product: {data}")
        response = requests.post(endpoint_url, json=data, headers=headers)
        if not response.ok:
            raise Exception(f"Error getting product {product_variation.sku} - {product_variation.code}\n {response.text}")
        if response.status_code == 204:
            raise Exception("Status code is 204, meaning nothing was updated or created")
        
        _logger.info(f"response text: {response.text}")
        response_data = response.json()
        _logger.debug(f"Response data: {response_data}")
        if isinstance(response_data, list):
            res = ListedSynchronizeResponse(results=response_data)
            if res.results and hasattr(res.results[0], 'error_context') and res.results[0].error_context:
                raise Exception(f"Errored while updating {res.results[0].error_context} {res.model_dump_json()}")
            return res.results[0] if res.results else None

        if isinstance(response_data, dict) and 'name' in response_data:
            return Product(**response_data)

        if isinstance(response_data, dict) and any(key in response_data for key in ['product', 'variation', 'statusCode']):
            sync_resp = SynchronizeResponse(**response_data)
            if sync_resp.error_context:
                raise Exception(f"Errored while updating {sync_resp.error_context} - {sync_resp.model_dump_json()}")
            return sync_resp

        if isinstance(response_data, dict) and 'message' in response_data:
            error_res = ErrorReason(**response_data)
            raise Exception(f"Errored with the following message {error_res.message} - {error_res.model_dump_json()}")
        
        raise Exception(f"Unhandled response format, check response {response.text}")

    def get(self, product_id: int) -> "ProductService":
        endpoint_url = self.config.get_endpoint(f'{self.endpoint}/{product_id}')
        headers = self.config.headers
        response = requests.get(endpoint_url, headers=headers)
        if not response.ok:
            raise Exception(f"Error getting product {product_id}\n {response.text}")
        response_data = response.json()
        return self(**response_data)

    def get_bundle(self, product_id: int) -> BundleResponse:
        endpoint_url = self.config.get_endpoint(f'{self.endpoint}/{product_id}/bundles')
        headers = self.config.headers
        response = requests.get(endpoint_url, headers=headers)
        if not response.ok:
            raise Exception(f"Error getting bundle {product_id}\n {response.text}")
        return BundleResponse(**response.json())

    def get_ml_integration(self, product_id: int) -> MeliProduct:
        endpoint_url = self.config.get_endpoint(f'{self.endpoint}/{product_id}/listingintegration')
        headers = self.config.headers
        response = requests.get(endpoint_url, headers=headers)
        if not response.ok:
            raise Exception(f"Error getting ml integration {product_id}\n {response.text}")
        response_data = response.json()
        return MeliProduct(**response_data)

    def search(self, params: SearchProductParams) -> SearchProduct:
        endpoint: str = f'search/{self.endpoint}'
        headers = self.config.headers
        url = self.config.get_endpoint(endpoint)
        response = requests.get(url, headers=headers, params=clean_model_dump(params))
        if not response.ok:
            raise Exception(f"error in searching products {response.text}")
        return SearchProduct(**response.json())



from typing import List, Optional, Union
from pydantic import BaseModel, Field


class FacetValue(BaseModel):
    count: int
    value: Optional[Union[str, bool]] = None
    label: Union[str, bool]


class Facet(BaseModel):
    key: str
    value: List[FacetValue]
    is_collection: Optional[bool] = False
    translate: bool


class SearchStocks(BaseModel):
    warehouse: str
    quantity: int
    reserved: int


class SearchPrices(BaseModel):
    price_list_id: int = Field(..., alias='priceListId')
    price_list: str = Field(..., alias='priceList') 
    amount: float
    currency: str


class SearchIntegration(BaseModel):
    app: Optional[int] = None
    integration_id: Optional[str] = Field(None, alias='integrationId')
    permalink: Optional[str] = None
    status: Optional[str] = None
    listing_type: Optional[str] = Field(None, alias='listingType')
    safety_stock: Optional[int] = Field(None, alias='safetyStock')
    synchronize_stock: Optional[bool] = Field(None, alias='synchronizeStock')
    is_active: Optional[bool] = Field(None, alias='isActive')
    is_active_or_paused: Optional[bool] = Field(None, alias='isActiveOrPaused')
    id: Optional[int] = None


class SearchDeals(BaseModel):
    campaign: str
    product: int
    variation: str
    deal_price: float
    discount: float
    regular_price: float
    enabled: bool
    currency: str
    id: str


class SearchResultItem(BaseModel):
    search_score: float = Field(..., alias='@search.score')
    id: int
    product_id: Optional[int] = Field(None, alias='productId')
    company_id: Optional[int] = Field(None, alias='companyId')
    name:Optional[str] = None
    code: Optional[str] = None
    skus: List[str]
    brand: Optional[str] = None
    category: Optional[str] = None
    thumbnail: Optional[str] = None
    stocks: Optional[Union[List[SearchStocks], List]] = None
    warehouses_with_stock: Optional[List[str]] = Field(None, alias='warehousesWithStock')
    total_stock: Optional[int] = Field(None, alias='totalStock')
    has_pictures: Optional[bool] = Field(None, alias='hasPictures')
    buying_price: Optional[float] = Field(None, alias='buyingPrice')
    prices: Optional[Union[List[SearchPrices], List]] = None
    integration_ids: Optional[List[str]] = Field(None, alias='integrationIds')
    integration_apps: Optional[List[str]] = Field(None, alias='integrationApps')
    integrations: Optional[Union[List[SearchIntegration], List]] = None
    campaigns: Optional[List[str]] = None
    app: Optional[int] = None
    status: Optional[str] = None
    synchronize_stock: Optional[bool] = Field(None, alias='synchronizeStock')
    listing_type: Optional[str] = Field(None, alias='listingType')
    price_amount: Optional[float] = Field(None, alias='priceAmount')
    price_currency: Optional[str] = Field(None, alias='priceCurrency')
    category_id: Optional[str] = Field(None, alias='categoryId')
    category_base_id: Optional[str] = Field(None, alias='categoryBaseId')
    category_l1: Optional[str] = Field(None, alias='categoryL1')
    category_l2: Optional[str] = Field(None, alias='categoryL2')
    category_l3: Optional[str] = Field(None, alias='categoryL3')
    category_l4: Optional[str] = Field(None, alias='categoryL4')
    category_l5: Optional[str] = Field(None, alias='categoryL5')
    category_l6: Optional[str] = Field(None, alias='categoryL6')
    has_category: Optional[bool] = Field(None, alias='hasCategory')
    category_fixed: Optional[bool] = Field(None, alias='categoryFixed')
    accepts_mercadoenvios: Optional[bool] = Field(None, alias='acceptsMercadoenvios')
    shipping_mode: Optional[str] = Field(None, alias='shippingMode')
    local_pickup: Optional[bool] = Field(None, alias='localPickup')
    mandatory_free_shipping: Optional[bool] = Field(None, alias='mandatoryFreeShipping')
    free_shipping: Optional[bool] = Field(None, alias='freeShipping')
    free_shipping_cost: Optional[float] = Field(None, alias='freeShippingCost')
    template: Optional[str] = None
    youtube_id: Optional[str] = Field(None, alias='youtubeId')
    warranty: Optional[str] = None
    permalink: Optional[str] = None
    domain: Optional[str] = None
    attribute_completion_status: Optional[str] = Field(None, alias='attributeCompletionStatus')
    attribute_completion_count: Optional[int] = Field(None, alias='attributeCompletionCount')
    attribute_completion_total: Optional[int] = Field(None, alias='attributeCompletionTotal')
    deals: Optional[Union[SearchDeals, List]] = None
    campaign_status: Optional[List[str]] = Field(None, alias='campaignStatus')
    size_chart: Optional[str] = Field(None, alias='sizeChart')
    channel_status: Optional[List[str]] = Field(None, alias='channelStatus')
    channel_category_l1: Optional[List[str]] = Field(None, alias='channelCategoryL1')
    channel_category_l2: Optional[List[str]] = Field(None, alias='channelCategoryL2')
    channel_category_l3: Optional[List[str]] = Field(None, alias='channelCategoryL3')
    channel_category_id: Optional[List[str]] = Field(None, alias='channelCategoryId')
    channel_synchronizes_stock: Optional[List[str]] = Field(None, alias='channelSynchronizesStock')
    channel_has_category: Optional[List[str]] = Field(None, alias='channelHasCategory')
    catalog_products_status: Optional[List[str]] = Field(None, alias='catalogProductsStatus')
    metadata: Optional[List[str]] = None
    integration_tags: Optional[List[str]] = Field(None, alias='integrationTags')
    variations_integration_ids: Optional[List[str]] = Field(None, alias='variationsIntegrationIds')
    channel_pictures_templates: Optional[List[str]] = Field(None, alias='channelPicturesTemplates')
    channel_pictures_templates_apps: Optional[List[str]] = Field(None, alias='channelPicturesTemplatesApps')


class SearchProduct(BaseModel):
    count: int
    facets: List[Facet]
    results: List[SearchResultItem]


class SearchProductParams(BaseModel):
    top: Optional[int]
    skip: Optional[int]
    filter: Optional[str] = Field(default=None, alias='$filter')
    search: Optional[str] = Field(default=None)
    sales_channel: Optional[str] = Field(default='2', alias='salesChannel')



from pydantic import BaseModel, Field
from typing import List, Optional
import requests
from producteca.abstract.abstract_dataclass import BaseService
from producteca.sales_orders.search_sale_orders import SearchSalesOrderParams, SearchSalesOrder
from producteca.payments.payments import Payment
from producteca.shipments.shipment import Shipment
from producteca.utils import clean_model_dump
from dataclasses import dataclass
import logging
_logger = logging.getLogger(__name__)


class SaleOrderLocation(BaseModel):
    street_name: Optional[str] = Field(None, alias="streetName")
    street_number: Optional[str] = Field(None, alias="streetNumber") 
    address_notes: Optional[str] = Field(None, alias="addressNotes")
    state: Optional[str] = None
    city: Optional[str] = None
    neighborhood: Optional[str] = None
    zip_code: Optional[str] = Field(None, alias="zipCode")


class SaleOrderBillingInfo(BaseModel):
    doc_type: Optional[str] = Field(None, alias="docType")
    doc_number: Optional[str] = Field(None, alias="docNumber")
    street_name: Optional[str] = Field(None, alias="streetName")
    street_number: Optional[str] = Field(None, alias="streetNumber")
    comment: Optional[str] = None
    zip_code: Optional[str] = Field(None, alias="zipCode")
    city: Optional[str] = None
    state: Optional[str] = None
    state_registration: Optional[str] = Field(None, alias="stateRegistration")
    tax_payer_type: Optional[str] = Field(None, alias="taxPayerType")
    first_name: Optional[str] = Field(None, alias="firstName")
    last_name: Optional[str] = Field(None, alias="lastName")
    business_name: Optional[str] = Field(None, alias="businessName")


class SaleOrderProfile(BaseModel):
    app: int
    integration_id: str = Field(alias="integrationId")
    nickname: Optional[str] = None


class SaleOrderContact(BaseModel):
    id: int
    name: str
    contact_person: Optional[str] = Field(None, alias="contactPerson")
    mail: Optional[str] = None
    phone_number: Optional[str] = Field(None, alias="phoneNumber")
    tax_id: Optional[str] = Field(None, alias="taxId")
    location: Optional[SaleOrderLocation] = None
    notes: Optional[str] = None
    type: Optional[str] = None
    price_list: Optional[str] = Field(None, alias="priceList")
    price_list_id: Optional[str] = Field(None, alias="priceListId")
    profile: Optional[SaleOrderProfile] = None
    billing_info: Optional[SaleOrderBillingInfo] = Field(None, alias="billingInfo")


class SaleOrderIntegrationId(BaseModel):
    alternate_id: Optional[str] = Field(None, alias="alternateId")
    integration_id: str = Field(alias="integrationId")
    app: int


class SaleOrderVariationPicture(BaseModel):
    url: str
    id: Optional[int] = None


class SaleOrderVariationStock(BaseModel):
    warehouse_id: Optional[int] = Field(None, alias="warehouseId")
    warehouse: str
    quantity: int
    reserved: int
    last_modified: Optional[str] = Field(None, alias="lastModified")
    available: int


class SaleOrderVariationAttribute(BaseModel):
    key: str
    value: str


class SaleOrderVariation(BaseModel):
    supplier_code: Optional[str] = Field(None, alias="supplierCode")
    pictures: Optional[List[SaleOrderVariationPicture]] = None
    stocks: Optional[List[SaleOrderVariationStock]] = None
    integration_id: Optional[int] = Field(None, alias="integrationId")
    attributes_hash: Optional[str] = Field(None, alias="attributesHash")
    primary_color: Optional[str] = Field(None, alias="primaryColor")
    secondary_color: Optional[str] = Field(None, alias="secondaryColor")
    size: Optional[str] = None
    thumbnail: Optional[str] = None
    attributes: Optional[List[SaleOrderVariationAttribute]] = None
    integrations: Optional[List[SaleOrderIntegrationId]] = None
    id: int
    sku: str
    barcode: Optional[str] = None


class SaleOrderProduct(BaseModel):
    name: str
    code: str
    brand: Optional[str] = None
    id: int


class SaleOrderQuestion(BaseModel):
    text: Optional[str] = None
    answer: Optional[str] = None


class SaleOrderConversation(BaseModel):
    questions: Optional[List[SaleOrderQuestion]] = None


class SaleOrderLine(BaseModel):
    price: float
    original_price: Optional[float] = Field(None, alias="originalPrice")
    transaction_fee: Optional[float] = Field(None, alias="transactionFee")
    product: SaleOrderProduct
    variation: SaleOrderVariation
    order_variation_integration_id: Optional[str] = Field(None, alias="orderVariationIntegrationId")
    quantity: int
    conversation: Optional[SaleOrderConversation] = None
    reserved: Optional[int] = None
    id: int


class SaleOrderCard(BaseModel):
    payment_network: Optional[str] = Field(None, alias="paymentNetwork")
    first_six_digits: Optional[int] = Field(None, alias="firstSixDigits")
    last_four_digits: Optional[int] = Field(None, alias="lastFourDigits")
    cardholder_identification_number: Optional[str] = Field(None, alias="cardholderIdentificationNumber")
    cardholder_identification_type: Optional[str] = Field(None, alias="cardholderIdentificationType")
    cardholder_name: Optional[str] = Field(None, alias="cardholderName")


class SaleOrderPaymentIntegration(BaseModel):
    integration_id: str = Field(alias="integrationId")
    app: int


class SaleOrderPayment(BaseModel):
    date: Optional[str] = None
    amount: float
    coupon_amount: Optional[float] = Field(None, alias="couponAmount")
    status: Optional[str] = None
    method: Optional[str] = None
    integration: Optional[SaleOrderPaymentIntegration] = None
    transaction_fee: Optional[float] = Field(None, alias="transactionFee")
    installments: Optional[int] = None
    card: Optional[SaleOrderCard] = None
    notes: Optional[str] = None
    authorization_code: Optional[str] = Field(None, alias="authorizationCode")
    has_cancelable_status: Optional[bool] = Field(None, alias="hasCancelableStatus")
    id: Optional[int] = None


class SaleOrderShipmentMethod(BaseModel):
    tracking_number: Optional[str] = Field(None, alias="trackingNumber")
    tracking_url: Optional[str] = Field(None, alias="trackingUrl")
    courier: Optional[str] = None
    mode: Optional[str] = None
    cost: Optional[float] = None
    type: Optional[str] = None
    eta: Optional[int] = None
    status: Optional[str] = None


class SaleOrderShipmentProduct(BaseModel):
    product: int
    variation: int
    quantity: int


class SaleOrderShipmentIntegration(BaseModel):
    app: int
    integration_id: str = Field(alias="integrationId")
    status: str
    id: int


class SaleOrderShipment(BaseModel):
    date: str
    products: List[SaleOrderShipmentProduct]
    method: Optional[SaleOrderShipmentMethod] = None
    integration: Optional[SaleOrderShipmentIntegration] = None
    receiver: Optional[dict] = None
    id: int


class SaleOrderInvoiceIntegrationAbstract(BaseModel):
    id: Optional[int] = None
    integration_id: Optional[str] = Field(None, alias="integrationId")
    app: Optional[int] = None
    created_at: Optional[str] = Field(None, alias="createdAt")
    decrease_stock: Optional[bool] = Field(None, alias="decreaseStock")


class SaleOrderInvoiceIntegration(SaleOrderInvoiceIntegrationAbstract):
    document_url: Optional[str] = Field(None, alias="documentUrl")
    xml_url: Optional[str] = Field(None, alias="xmlUrl")


class SaleOrderInvoiceIntegrationPut(SaleOrderInvoiceIntegrationAbstract):
    document_url: Optional[str] = Field(None, alias="documentUrl")
    xml_url: Optional[str] = Field(None, alias="xmlUrl")

class SaleOrderWarehouseIntegration(BaseModel):
    app: Optional[int] = None
    status: Optional[str] = None
    integration_id: Optional[str] = Field(None, alias="integrationId")


class SaleOrder(BaseModel):
    tags: Optional[List[str]] = None
    integrations: Optional[List[SaleOrderIntegrationId]] = None
    invoice_integration: Optional[SaleOrderInvoiceIntegration] = Field(None, alias="invoiceIntegration")
    channel: Optional[str] = None
    pii_expired: Optional[bool] = Field(None, alias="piiExpired")
    contact: Optional[SaleOrderContact] = None
    lines: Optional[List[SaleOrderLine]] = None
    warehouse: Optional[str] = None
    warehouse_id: Optional[int] = Field(None, alias="warehouseId")
    warehouse_integration: Optional[SaleOrderWarehouseIntegration] = Field(None, alias="warehouseIntegration")
    pick_up_store: Optional[str] = Field(None, alias="pickUpStore")
    payments: Optional[List[SaleOrderPayment]] = None
    shipments: Optional[List[SaleOrderShipment]] = None
    amount: Optional[float] = None
    shipping_cost: Optional[float] = Field(None, alias="shippingCost")
    financial_cost: Optional[float] = Field(None, alias="financialCost")
    paid_approved: Optional[float] = Field(None, alias="paidApproved")
    payment_status: Optional[str] = Field(None, alias="paymentStatus")
    delivery_status: Optional[str] = Field(None, alias="deliveryStatus")
    payment_fulfillment_status: Optional[str] = Field(None, alias="paymentFulfillmentStatus")
    delivery_fulfillment_status: Optional[str] = Field(None, alias="deliveryFulfillmentStatus")
    delivery_method: Optional[str] = Field(None, alias="deliveryMethod")
    payment_term: Optional[str] = Field(None, alias="paymentTerm")
    currency: Optional[str] = None
    custom_id: Optional[str] = Field(None, alias="customId")
    is_open: Optional[bool] = Field(None, alias="isOpen")
    is_canceled: Optional[bool] = Field(None, alias="isCanceled")
    cart_id: Optional[str] = Field(None, alias="cartId")
    draft: Optional[bool] = None
    promise_delivery_date: Optional[str] = Field(None, alias="promiseDeliveryDate")
    promise_dispatch_date: Optional[str] = Field(None, alias="promiseDispatchDate")
    has_any_shipments: Optional[bool] = Field(None, alias="hasAnyShipments")
    has_any_payments: Optional[bool] = Field(None, alias="hasAnyPayments")
    date: Optional[str] = None
    notes: Optional[str] = None
    id: int


class SaleOrderSynchronize(BaseModel):
    id: int
    invoice_integration: SaleOrderInvoiceIntegration = Field(alias="invoiceIntegration")
    notes: Optional[str] = None
    tags: Optional[List[str]] = None


class UpdateStatus(BaseModel):
    updated: bool = False


class SaleOrderSyncResponse(BaseModel):
    basic: UpdateStatus = Field(default_factory=UpdateStatus)
    contact: UpdateStatus = Field(default_factory=UpdateStatus)
    shipments: UpdateStatus = Field(default_factory=UpdateStatus) 
    payments: UpdateStatus = Field(default_factory=UpdateStatus)
    invoice_integration: UpdateStatus = Field(alias="invoiceIntegration", default_factory=UpdateStatus)


@dataclass
class SaleOrderService(BaseService):
    endpoint: str = 'salesorders'

    def __call__(self, **payload):
        self._record = SaleOrder(**payload)
        return self

    def __repr__(self):
        return repr(self._record)
    
    def get(self, sale_order_id: int) -> "SaleOrderService":
        endpoint = f'{self.endpoint}/{sale_order_id}'
        url = self.config.get_endpoint(endpoint)
        response = requests.get(url, headers=self.config.headers)
        if not response.ok:
            raise Exception(f"Order {sale_order_id} could not be fetched. Error {response.status_code} {response.text}")
        response_data = response.json()
        return self(**response_data)

    def get_shipping_labels(self):
        if not self._record:
            raise Exception("You need to add a record id")
        endpoint = f'{self.endpoint}/{self._record.id}/labels'
        url = self.config.get_endpoint(endpoint)
        response = requests.get(url, headers=self.config.headers)
        if not response.ok:
            raise Exception("labels could not be gotten")
        return response.json()

    def close(self):
        if not self._record:
            raise Exception("You need to add a record id")
        endpoint = f'{self.endpoint}/{self._record.id}/close'
        url = self.config.get_endpoint(endpoint)
        response = requests.post(url, headers=self.config.headers)
        if not response.ok:
            raise Exception("Order could not be closed")

    def cancel(self):
        if not self._record:
            raise Exception("You need to add a record id")
        endpoint = f'{self.endpoint}/{self._record.id}/cancel'
        url = self.config.get_endpoint(endpoint)
        response = requests.post(url, headers=self.config.headers)
        if not response.ok:
            raise Exception("Order could not be cancelled")

    def synchronize(self) -> "SaleOrderService":
        if not self._record:
            raise Exception("You need to add a record by calling the resource and adding info")
        endpoint = f'{self.endpoint}/synchronize'
        url = self.config.get_endpoint(endpoint)
        # TODO: Check what can we sync, and what can we not sync
        sync_body = SaleOrderSynchronize(**clean_model_dump(self._record))
        response = requests.post(url, json=clean_model_dump(sync_body), headers=self.config.headers)
        if not response.ok:
            raise Exception(f"Synchronize error {response.status_code} {response.text}")
        sync_res = SaleOrderSyncResponse(**response.json()) # noqa
        return self

    def invoice_integration(self):
        if not self._record:
            raise Exception("You need to add a record id")
        if not self._record.invoice_integration:
            raise Exception(f"Sale Order {self._record.id} does not have an invoiceIntegration")
        
        invoice_integration_data = clean_model_dump(self._record.invoice_integration)
        
        if self._record.invoice_integration.id:
            endpoint = f'{self.endpoint}/{self._record.id}/invoiceIntegration'
            url = self.config.get_endpoint(endpoint)
            response = requests.put(url, headers=self.config.headers,
                                    json={"id": self._record.id,
                                          "invoiceIntegration": invoice_integration_data})
        else:
            endpoint = f'{self.endpoint}/synchronize'
            url = self.config.get_endpoint(endpoint)
            response = requests.post(url, headers=self.config.headers,
                                     json={"id": self._record.id,
                                           "invoiceIntegration": invoice_integration_data})
        
        if not response.ok:
            raise Exception(f"Error on resposne {response.text}")
        return response.ok

    def search(self, params: SearchSalesOrderParams):
        endpoint: str = f"search/{self.endpoint}"
        headers = self.config.headers
        url = self.config.get_endpoint(endpoint)
        new_url = f"{url}?$filter={params.filter}&top={params.top}&skip={params.skip}"
        response = requests.get(
            new_url,
            headers=headers,
        )
        if not response.ok:
            raise Exception(f"Error on resposne {response.status_code} - {response.text}")
        response_data = response.json()
        return SearchSalesOrder(**response_data)

    def add_payment(self, payload) -> Payment:
        if not self._record:
            raise Exception("You need to add a record id")
        payment = Payment(**payload)
        url = self.config.get_endpoint(f"{self.endpoint}/{self._record.id}/payments")
        res = requests.post(url, json=clean_model_dump(payment), headers=self.config.headers)
        if not res.ok:
            raise Exception(f"Error on resposne {res.text}")
        return Payment(**res.json())

    def update_payment(self, payment_id: int, payload) -> "Payment":
        if not self._record:
            raise Exception("You need to add a record id")
        payment = Payment(**payload)
        url = self.config.get_endpoint(f"{self.endpoint}/{self._record.id}/payments/{payment_id}")
        res = requests.put(url, json=clean_model_dump(payment), headers=self.config.headers)
        if not res.ok:
            raise Exception(f"Error on payment update {res.text}")
        return Payment(**res.json())

    def add_shipment(self, payload) -> "Shipment":
        if not self._record:
            raise Exception("You need to add a record id")
        shipment = Shipment(**payload)
        url = self.config.get_endpoint(f"{self.endpoint}/{self._record.id}/shipments")
        res = requests.post(url, json=clean_model_dump(shipment), headers=self.config.headers)
        if not res.ok:
            raise Exception(f"Error on shipment add {res.text}")
        return Shipment(**res.json())

    def update_shipment(self, shipment_id: str, payload) -> "Shipment":
        if not self._record:
            raise Exception("You need to add a record id")
        shipment = Shipment(**payload)
        url = self.config.get_endpoint(f"{self.endpoint}/{self._record.id}/shipments/{shipment_id}")
        res = requests.put(url, json=clean_model_dump(shipment), headers=self.config.headers)
        if not res.ok:
            raise Exception(f"Error on shipment update {res.text}")
        return Shipment(**res.json())



"""
Utility functions for the producteca package
"""
from typing import Any, Dict, List, Union
from pydantic import BaseModel


def exclude_empty_values(obj: Any) -> Any:
    """Recursively remove None, empty lists, empty strings, and empty dicts
    
    Note: Preserves 0 values for numeric fields as they are valid prices/quantities
    """
    if isinstance(obj, dict):
        filtered_dict = {}
        for k, v in obj.items():
            # Skip None, empty lists, empty strings, empty dicts
            if v is None or v == [] or v == "" or v == {}:
                continue
            # Keep numeric 0 values - they are valid for prices, quantities, etc.
            filtered_dict[k] = exclude_empty_values(v)
        return filtered_dict
    elif isinstance(obj, list):
        filtered_list = [exclude_empty_values(item) for item in obj if item is not None]
        return [item for item in filtered_list if item != [] and item != "" and item != {}]
    else:
        return obj


def clean_model_dump(model: BaseModel, by_alias: bool = True, exclude_none: bool = True, **kwargs) -> Dict[str, Any]:
    """
    Enhanced model_dump that automatically cleans empty values
    
    Args:
        model: Pydantic model instance
        by_alias: Use field aliases in output
        exclude_none: Exclude None values
        **kwargs: Additional arguments to pass to model_dump
    
    Returns:
        Clean dictionary with empty values removed
    """
    raw_data = model.model_dump(by_alias=by_alias, exclude_none=exclude_none, **kwargs)
    return exclude_empty_values(raw_data)


from abc import ABC
from ..config.config import ConfigProducteca
from ..utils import clean_model_dump
from dataclasses import dataclass
from typing import Optional, Any


@dataclass
class BaseService(ABC):
    config: ConfigProducteca
    endpoint: str
    _record: Optional[Any] = None
    
    def __repr__(self):
        return repr(self._record)

    def to_dict(self):
        return clean_model_dump(self._record)

    def to_json(self):
        import json
        return json.dumps(clean_model_dump(self._record))

    def __getattr__(self, key):
        return getattr(self._record, key)



from pydantic import BaseModel
from typing import Optional


class PaymentCard(BaseModel):
    paymentNetwork: Optional[str] = None
    firstSixDigits: Optional[int] = None
    lastFourDigits: Optional[int] = None
    cardholderIdentificationNumber: Optional[str] = None
    cardholderIdentificationType: Optional[str] = None
    cardholderName: Optional[str] = None


class PaymentIntegration(BaseModel):
    integrationId: str
    app: int


class Payment(BaseModel):
    date: str
    amount: float
    couponAmount: Optional[float] = None
    status: str
    method: str
    integration: Optional[PaymentIntegration] = None
    transactionFee: Optional[float] = None
    installments: Optional[int] = None
    card: Optional[PaymentCard] = None
    notes: Optional[str] = None
    hasCancelableStatus: bool
    id: Optional[int] = None





import unittest
from unittest.mock import patch, Mock
from producteca.sales_orders.sales_orders import SaleOrder
from producteca.client import ProductecaClient


class TestSaleOrder(unittest.TestCase):

    def setUp(self):
        self.client = ProductecaClient(token="test_client", api_key="test_secret")
        self.sale_order_id = 123
        self.mock_response = {
            "id": self.sale_order_id,
            "contact": {"id": 1, "name": "Test Contact"},
            "lines": [],
            "invoiceIntegration": {
                'id': 1,
                'integrationId': 'test-integration',
                'app': 1,
                'createdAt': '2023-01-01',
                'decreaseStock': True
            }
        }

    @patch('requests.get')
    def test_get_sale_order(self, mock_get):
        mock_get.return_value = Mock(
            status_code=200,
            json=lambda: self.mock_response
        )

        sale_order = self.client.SalesOrder.get(self.sale_order_id)
        self.assertEqual(sale_order.id, self.sale_order_id)
        mock_get.assert_called_once()

    @patch('requests.get')
    def test_get_shipping_labels(self, mock_get):
        mock_labels = ["label1", "label2"]
        mock_get.return_value = Mock(
            status_code=200,
            json=lambda: mock_labels
        )

        labels = self.client.SalesOrder(id=1234, invoiceIntegration={
                'id': 1,
                'integrationId': 'test-integration',
                'app': 1,
                'createdAt': '2023-01-01',
                'decreaseStock': True,
                "documentUrl": "https://aallala.copm",
                "xmlUrl": "https://aallala.copm",
            }).get_shipping_labels()
        self.assertEqual(labels, mock_labels)
        mock_get.assert_called_once()

    @patch('requests.post')
    def test_close_sale_order(self, mock_post):
        mock_post.return_value = Mock(
            status_code=200
        )

        self.client.SalesOrder(id=1234, invoiceIntegration={
                'id': 1,
                'integrationId': 'test-integration',
                'app': 1,
                'createdAt': '2023-01-01',
                'decreaseStock': True,
                "documentUrl": "https://aallala.copm",
                "xmlUrl": "https://aallala.copm",
            }).close()
        mock_post.assert_called_once()

    @patch('requests.post')
    def test_cancel_sale_order(self, mock_post):
        mock_post.return_value = Mock(
            status_code=200,
            json=lambda: {"status": "cancelled"}
        )

        self.client.SalesOrder(id=1234, invoiceIntegration={
                'id': 1,
                'integrationId': 'test-integration',
                'app': 1,
                'createdAt': '2023-01-01',
                'decreaseStock': True,
                "documentUrl": "https://aallala.copm",
                "xmlUrl": "https://aallala.copm",
            }).cancel()
        mock_post.assert_called_once()

    @patch('requests.post')
    def test_synchronize_sale_order(self, mock_post):
        sale_order = SaleOrder(**self.mock_response)
        mock_post.return_value = Mock(
            status_code=200,
            json=lambda: self.mock_response
        )

        response = self.client.SalesOrder(**sale_order.model_dump(by_alias=True)).synchronize()
        self.assertEqual(response.id, self.sale_order_id)
        mock_post.assert_called_once()

    @patch('requests.put')
    def test_invoice_integration(self, mock_put):
        invoice_data = {
                'id': 1,
                'integrationId': 'test-integration',
                'app': 1,
                'createdAt': '2023-01-01',
                'decreaseStock': True,
                "documentUrl": "https://aallala.copm",
                "xmlUrl": "https://aallala.copm",
            }

        mock_put.return_value = Mock(
            status_code=200,
            json=lambda: invoice_data,
            ok=True
        )

        response = self.client.SalesOrder(id=self.sale_order_id, invoiceIntegration=invoice_data).invoice_integration()
        self.assertTrue(response)
        mock_put.assert_called_once()


if __name__ == '__main__':
    unittest.main()



import unittest
from unittest.mock import patch, Mock
from producteca.products.products import Product
from producteca.client import ProductecaClient


class TestProduct(unittest.TestCase):
    def setUp(self):
        self.client = ProductecaClient(token="test_client_id", api_key="test_client_secret")
        self.test_product = Product(
            sku="TEST001",
            name="Test Product",
            code="TEST001",
            category="Test"
        )
        self.product_to_create_payload = {
            "sku": "9817234",
            "code": "871234",
            "name": "Hola test",
            "buyingPrice": 0,
            "deals": [
                {
                "campaign": "string",
                "regularPrice": 0,
                "dealPrice": 0
                }
            ],
                "prices": [
                {
                "amount": 10,
                "currency": "Local",
                "priceList": "Default"
                }
            ],
                "stocks": [
                    {
                    "quantity": 2,
                    "availableQuantity": 2,
                    "warehouse": "Default"
                    }
                ],
            }

    @patch('requests.post')
    def test_create_product_success(self, mock_post):
        # Mock successful response
        mock_response = Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = self.test_product.model_dump()
        mock_post.return_value = mock_response

        response = self.client.Product(**self.test_product.model_dump()).synchronize(self.product_to_create_payload)
        
        self.assertEqual(response.sku, "TEST001")

    @patch('requests.post')
    def test_create_product_not_exist(self, mock_post):
        # Mock product not found response
        mock_response = Mock()
        mock_response.status_code = 204
        mock_post.return_value = mock_response

        with self.assertRaises(Exception):
            self.client.Product.synchronize(self.product_to_create_payload)

    @patch('requests.post')
    def test_update_product_success(self, mock_post):
        payload = self.product_to_create_payload
        mock_response = Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = self.test_product.model_dump()
        mock_post.return_value = mock_response

        response = self.client.Product(**self.test_product.model_dump()).synchronize(payload)
        
        self.assertEqual(response.name, "Test Product")

    @patch('requests.get')
    def test_get_product(self, mock_get):
        # Mock get product response
        mock_response = Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = self.test_product.model_dump()
        mock_get.return_value = mock_response

        response = self.client.Product.get(1)
        
        self.assertEqual(response.sku, "TEST001")

    @patch('requests.get')
    def test_get_bundle(self, mock_get):
        # Mock get bundle response
        mock_response = Mock()
        mock_response.status_code = 200
        test_prod = {
                "results": [
                    {
                    "companyId": 0,
                    "productId": 0,
                    "variations": [
                        {
                        "variationId": 0,
                        "components": [
                            {
                            "quantity": 0,
                            "variationId": 0,
                            "productId": 0
                            }
                        ]
                        }
                    ],
                    "id": "string"
                    }
                ],
                "count": 0
                }
        mock_response.json.return_value = test_prod
        mock_get.return_value = mock_response

        product = self.client.Product.get_bundle(1)
        
        self.assertEqual(product.count, 0)

    @patch('requests.get')
    def test_get_ml_integration(self, mock_get):
        # Mock ML integration response
        mock_response = Mock()
        mock_response.status_code = 200
        meli_product = {
            "hasCustomShippingCosts": True,
            "productId": 0,
            "shipping": {
                "localPickup": True,
                "mode": "string",
                "freeShipping": True,
                "freeShippingCost": 0,
                "mandatoryFreeShipping": True,
                "freeShippingMethod": "string"
            },
            "mShopsShipping": {
                "enabled": True
            },
            "addFreeShippingCostToPrice": True,
            "category": {
                "meliId": "string",
                "acceptsMercadoenvios": True,
                "suggest": True,
                "fixed": True
            },
            "attributeCompletion": {
                "productIdentifierStatus": "Complete",
                "dataSheetStatus": "Complete",
                "status": "Complete",
                "count": 0,
                "total": 0
            },
            "catalogProducts": [
                "string"
            ],
            "warranty": "string",
            "domain": "string",
            "listingTypeId": "GoldSpecial",
            "catalogProductsStatus": "Unlinked",
            "tags": [
                "string"
            ]
            }
        
        mock_response.json.return_value = meli_product
        mock_get.return_value = mock_response

        product = self.client.Product.get_ml_integration(1)
        
        self.assertEqual(product.listing_type_id, "GoldSpecial")


if __name__ == '__main__':
    unittest.main()
